function DataOut=TEprepare(varargin)

% TEPREPARE this function checks the input data and parameter for
% completeness and correctness. Further, it optimizes the embedding
% parameters and adds a substructure to the data, which is nesseccary for
% the further functions.
% TEPREPARE has to be performed on all dataset first!!!
%
% You can call this function directly as follows:
%         TEprepare(cfg, data)
%
% * DEPENDENCIES
%     - Package TSTOOL is used at nearest neighbors searches
%       required for the KSG estimator. (Gnu Public License)
%       http://www.dpi.physik.uni-goettingen.de/tstool/
%     - The following Matlab toolboxes:
%         - signal processing toolbox
%         - statistic toolbox
%     - The functions
%         - TEactdetect
%         - TEchannelselect
%         - TEtrialselect
%         - TEwait
%         - TEprepareview
%
%
% * INPUT PARAMETERS
%
%   data           = Fieldtrip raw data structure - it MUST contain:
%       .trial    = cell array (nr of channels x nr of samples) containing
%                    the data for each trial
%       .time      = cell (1xnr of samples) containing the time indices for
%                    each trial (in seconds)
%       .label     = cell (1xnr of channels), containing the labels
%                    (strings) of channels included in the data
%       .fsample   = value of sampling rate (in Hertz)
%
% AND
%
%   cfg: The configuration MUST contain:
%
%  cfg.sgncmb      = list of channelpairs
%                    cell array (Nx(source, target))
%  or
%  cfg.channel     = list of channels testing all by all
%
%  and
%
%  cfg.Path2TSTOOL = Path to the folder including the TSTOOL package
%  cfg.toi         = the time range of interest (vector 1 x numtoi) in s
%                    e.g. (time_from, time_to)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   WARNING:
%   The span of time needed for embedding is: (max(dim)-1)*max(tau)
%   The prediction time starts after this embedding time. Hence the span of
%   time defined in cfg.toi must be a good deal longer than the embedding
%   time, leastwise a multiple of the prediction time (nrk).
%
%       |<  embedding time  >|< prediction time ...
%   ----|--------------------|-----------------------------------|-->
%       |<                       cfg.toi                        >|
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%  cfg.TEcalctype =   'V' : prediction time is manually specified in
%                        cfg.predicttime_u
%                     'VW' : prediction time is tau and the putative source
%                        signal is shifted back cfg.predicttime_u.
%                        (to solve the problem of decreasing autoprediction
%                         for large prediction times)
%                     (default = 1)
%   cfg.predicttime_u = points ahead for the advance vector in ms
%                     (vector 1 x numu)
%
%  cfg.optimizemethod = Define method to optimize parameters: 'ragwitz' or
%                   'cao'
%
%  if you choose 'ragwitz':
%       cfg.ragdim  = range of embedding dimensions to scan vector
%                     (1xnumdim)
%       cfg.ragtaurange  = vector (1x2) of min and max embedding delays (in
%                          multiples of the autocorrelation decay time)
%       cfg.ragtausteps  = number of equidistant steps in ragtaurange
%                          (min 5) (default = 10)
%       cfg.flagNei = 'Range' or 'Mass' type of neighbor search
%       cfg.sizeNei = Radius or mass for the neighbor search according to
%                     flagNeighborhood
%       cfg.repPred = repPred represents the number of points for which the
%                     prediction is performed (it has to be smaller than
%                     length(timeSeries)-(dimEmb-1)*tauEmb-u)
%
%  if you choose 'cao':
%       cfg.caodim      = range of embedding dimension to scan with
%                       stepwidth 1 (vector 1 x numdim)
%                       (default = [1,2,3,4,5,6,7,8,9,10])
%       cfg.caokth_neighbors = number of neighbors for fixed mass search
%                    for cao (controls balance of bias/statistical errors)
%                    (default = 4)
%       cfg.tau         = embedding delay in units of ACT (x*ACT)
%                       (default = 1.5)
%
%   cfg.kth_neighbors = number of neighbors for fixed mass search (controls
%                     balance of bias/statistical errors) (default = 4)
%   cfg.TheilerT    = number of temporal neighbors excluded to avoid serial
%                     correlations (Theiler correction) (default = ACT)
%
%
%  cfg.trialselect = ACT threshholding of trials - 'ACT' ,'range' or 'no'
%                    (default = 'ACT')
%      if you chose 'ACT' (or nothing):
%      cfg.actthrvalue = max threshold for ACT; min threshold
%      cfg.minnrtrials = minimum Nr of trials with ACT < actthrest used to
%                    calculate transfer entropy
%      if you chose 'range':
%      cfg.trial_from  = Inferior limit for the trials to be considered
%      cfg.trial_to    = Superior limit for the trials to be considered
%  cfg.maxlag      = the range of lags for computing the auto correlation
%                    time: from -MAXLAG to MAXLAG (default = 1000)
%
%
% * OUTPUT PARAMETERS
%
%   DATA    = The output of this function is the data from the input with
%           the added structure TEprepare.  THE DATA IS NOT SAVED BY THIS
%           FUNCTION. You have to do it by yourself!
%   .TEprepare
%       .channelcombi       = matrix (nx2) with indices of channels in n
%                             user-defined channel combinations
%       .channelcombilabel  = cell with the channel labels of the
%                             channel pairs
%       .usedchannel        = vector with indices of all unique channels of
%                             the channel combinations
%       .usedchannellabel   = cell with the chanel labels of the unique
%                             channels
%       .ACT                = matrix (trialxchannel) with the values of
%                             the auto correlation decay times of the
%                             signals
%       .trials             = matrix (channelxtrial) with indices of the
%                             used trials
%       .nrtrials           = vector with the number of the used trials per
%                             channel combination
%       .nrreferencepoints  = matrix (channelxtrial) with the number of
%                             reference point for the cao calculation
%       .cao                = Structure containing two matrices
%                             (trial x channel x caodim) with the
%                             values E1 and E2 of the cao function
%       .optdimmat          = matrix (channelxtrial) with optimal
%                             embedding dimension
%       .optdim             = max of the optdimmat which should be used as
%                             embedding dimension in the further steps
%       .cfg                = cfg from the input
%
%
%       undocumented options cfg.TEcalctype ('V', 'VW')
% 
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation;
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY;
%
% Version 1.01 by Michael Lindner, Raul Vicente, Michael Wibral
% Frankfurt 2010
%

% CHANGELOG:
% 2011-0809: MW - changed allowed values of TEcalctype to V for Vicente
% style estimator and 'VW' for Wibral-Vicente style estimator. Made sure
% cfg.TEcalctype is passed on down to the level of transferentropy.m
%
% 2011-08-01: MW - tried to check and follow the bookeeping of channel
% indices - replaced channel with uc1(channel where necessary)
%


%% Remember the working directory
working_directory = pwd;


%% check data
% -------------------------------------------------------------------------
fprintf('\nCheck data and config\n');

if isfield(varargin{1},'toi') && isstruct(varargin{1}) && isstruct(varargin{2}) && isfield(varargin{2},'trial')
    cfg =  varargin{1};
    data = varargin{2};
else
    error('\nTRANSFER ENTROPY ERROR: incorrect input values, see help!');
end


% check data using checkdata from Fieldtrip
[data] = ft_checkdata(data, 'datatype','raw');

% check whether time axes and trials have the same number of entries
if iscell(data.time) % one time axis per trial
    for tt=1:size(data.trial,2) % for each trial
        if ~( size(data.time{tt},2) == size(data.trial{tt},2) )
            disp('in trial Nr: ')
            disp(num2str(tt))
            error('TRANSFER ENTROPY ERROR! incorrect number of samples in time axis or trial detected')
        end
    end
else % time is a single vector
    for tt=    size(data.trial,2)    % for each trial
        if ~( length(data.time) == size(data.trial{tt},2) )
            disp('in trial Nr: ')
            disp(num2str(tt))
            error('TRANSFER ENTROPY ERROR! incorrect number of samples in time axis or trial detected')
        end
    end
end

% check the data structure
if ~isfield(data, 'trial'),
    error('\nTRANSFER ENTROPY ERROR: data must be in ''.trial''-field, see help!');
end;
if ~isfield(data, 'time'),
    error('\nTRANSFER ENTROPY ERROR: data contains no ''.time''-field, see help!');
end;
if ~isfield(data, 'label'),
    error('\nTRANSFER ENTROPY ERROR: data contains no ''.label''-field, see help!');
end;
if ~isfield(data, 'fsample'),
    error('\nTRANSFER ENTROPY ERROR: data contains no ''.fsample''-field, see help!');
end;

% MW: is this for fMRI?
% ML: yes!
if isfield(data, 'datatype'),
    TEprepare.datatype = data.datatype;
end;

if size(data.time,1)>size(data.time,2)
    data.time=data.time';
end


% check configuration and set defaults
if ~isfield(cfg, 'trialselect'),      cfg.trialselect = 'ACT';         end;
if ~isfield(cfg, 'maxlag'),           cfg.maxlag = 1000;               end;
if ~isfield(cfg, 'TEcalctype'),       cfg.TEcalctype = 'V';            end;


% check optimizemethod
if ~isfield(cfg, 'optimizemethod'),  cfg.optimizemethod = 'ragwitz';  end;

if strcmp(cfg.optimizemethod, 'ragwitz')
    
    % check input for ragwitz
    
    if ~isfield(cfg, 'ragtausteps'), cfg.ragtausteps = 10;  end
    if isfield(cfg, 'ragtausteps')
        if cfg.ragtausteps < 5
            error('\nTRANSFER ENTROPY ERROR: cfg.ragtausteps must be 5 or higher, see help!')
        end
    end
    
    if ~isfield(cfg, 'ragtaurange'),
        error('\nTRANSFER ENTROPY ERROR: specify cfg.ragtaurange, see help!')
    else
        
        if cfg.ragtaurange(1) > cfg.ragtaurange(2)
            error('\nTRANSFER ENTROPY ERROR: cfg.ragtaurange must be a vector with increasing values!')
        end
        
        if size(cfg.ragtaurange,1)>size(cfg.ragtaurange,2)
            cfg.ragtaurange=cfg.ragtaurange';
        end
        
        if size(cfg.ragtaurange,2) > 2 || size(cfg.ragtaurange,2) < 2
            error('\nTRANSFER ENTROPY ERROR: cfg.ragtaurange has too many values, see help!')
        end
        
        % create cfg.tau vector
        %     cfg.ragtau = cfg.ragtaurange(1):(cfg.ragtaurange(2)-cfg.ragtaurange(1))/(cfg.ragtausteps-1):cfg.ragtaurange(2);
        cfg.ragtau = cfg.ragtaurange(1):(cfg.ragtaurange(2)-cfg.ragtaurange(1))/(cfg.ragtausteps):cfg.ragtaurange(2);
        
    end
    
    if ~isfield(cfg, 'ragdim'),
        cfg.ragdim = 1:10;
    else
        ragdimsort=sort(cfg.ragdim);
        if ragdimsort ~= cfg.ragdim
            error('\nTRANSFER ENTROPY ERROR: cfg.ragdim must be a vector with increasing values!')
        end
        
        if size(cfg.ragdim,1)>size(cfg.ragdim,2)
            cfg.ragdim=cfg.ragdim';
        end
    end;
    
    
elseif strcmp(cfg.optimizemethod, 'cao')
    
    if ~isfield(cfg, 'caokth_neighbors'), cfg.caokth_neighbors = 4;        end;
    if ~isfield(cfg, 'kth_neighbors'),  cfg.kth_neighbors = cfg.caokth_neighbors;  end;
    
    if ~isfield(cfg, 'tau'),              cfg.tau = 1.5;               end;
    % check input for cao
    if ~isfield(cfg, 'caodim'),
        cfg.caodim = [1,2,3,4,5,6,7,8,9,10];
    else
        
        l=length(cfg.caodim);
        diff=zeros(l-1,1);
        
        if l > 1
            for ii = 2:l
                diff(ii-1)=cfg.caodim(ii)-cfg.caodim(ii-1);
            end
        end
        
        if any(diff~=1)
            error('\nTRANSFER ENTROPY ERROR: cfg.caodim must contain a row of numbers with stepwith 1, see help!');
        end
        
        if length(cfg.caodim) < 5
            error('\nTRANSFER ENTROPY ERROR: cfg.caodim must contain at least 5 entries, see help!');
        end
    end;
    
    if size(cfg.caodim,1)>size(cfg.caodim,2)
        cfg.caodim=cfg.caodim';
    elseif size(cfg.tau,1)>size(cfg.tau,2)
        cfg.tau=cfg.tau';
    elseif size(cfg.caokth_neighbors,1)>size(cfg.caokth_neighbors,2)
        cfg.caokth_neighbors=cfg.caokth_neighbors';
    end;
end;


% check TE parameter
if isempty(cfg.predicttime_u),
    error('\nTRANSFER ENTROPY ERROR: specify cfg.predicttime_u, see help!');
elseif length(cfg.predicttime_u) > 1
    error('\nTRANSFER ENTROPY ERROR: cfg.predicttime_u must be a single value, see help!');
end



if ~isfield(cfg, 'kth_neighbors'),  cfg.kth_neighbors = 4;  end;

if ~isfield(cfg, 'TheilerT'),       cfg.TheilerT = 'ACT';   end;
if ~strcmp(cfg.TheilerT, 'ACT');
    if size(cfg.TheilerT,1)>1 || size(cfg.TheilerT,2)>1
        error('\nTRANSFER ENTROPY ERROR: cfg.TheilerT must contain a scalar, see help!');
    end
end


% check if channel or channelcombinations are defined
if ~isfield(cfg, 'channel') && ~isfield(cfg, 'sgncmb') ,
    error('\nTRANSFER ENTROPY ERROR: specify cfg.channel OR cfg.sgncmb, see help!');
elseif isfield(cfg, 'channel') && isfield(cfg, 'sgncmb') ,
    error('\nTRANSFER ENTROPY ERROR: specify cfg.channel OR cfg.sgncmb, see help!');
elseif isfield(cfg, 'channel') && ~isfield(cfg, 'sgncmb') ,
    if size(cfg.channel,2)>size(cfg.channel,1)
        cfg.channel=cfg.channel';
    end
    channelselect = 1;
    % MW: a warning becuase of some issue if only a subselection of
    % channels enters the analysis
    if max(size(cfg.channel))<size(data.trial{1},1) % If there are less channels in the channel field than in the data  !
       warning('\nTRANSFER ENTROPY WARNING: your are specifying a subselection of channels \n - please use cfg.sgncmb to specify channelcombinations directly'); 
    end
    
    
elseif ~isfield(cfg, 'channel') && isfield(cfg, 'sgncmb') ,
    if size(cfg.sgncmb) ~= 2
        error('\nTRANSFER ENTROPY ERROR: cfg.sgncmb has the wrong dimensions, see help!');
    end
    channelselect = 2;
end;


% check the format of input vectors
if size(cfg.toi,1)>2 || size(cfg.toi,1) >2
    error('\nTRANSFER ENTROPY ERROR: cfg.toi has more than two entries');
end

if size(cfg.toi,1)>size(cfg.toi,2)
    cfg.toi=cfg.toi';
end

fprintf(' - ok');

% check alternative ways of trial selection and the required related inputs
if strcmp(cfg.trialselect, 'ACT')
    if isempty(cfg.actthrvalue) || isempty(cfg.minnrtrials)
        error('\nTRANSFER ENTROPY ERROR: specify cfg.actthrvalue and cfg.minnrtrials for ACT thresholding, see help!');
    end
elseif strcmp(cfg.trialselect, 'range')
    if isempty(cfg.trial_from) || isempty(cfg.trial_to)
        error('\nTRANSFER ENTROPY ERROR: specify cfg.trial_from and cfg.trial_to for setting the range of used trials, see help!');
    end
end


%% read data
% -------------------------------------------------------------------------
fprintf('\nRead data');

datamat=zeros(size(data.trial,2),size(data.trial{1},1),size(data.trial{1},2));
for ii = 1:size(data.trial,2)
    datamat(ii,:,:)=data.trial{ii};
end


% read time values of the data
if iscell(data.time)
    alltime=cell2mat(data.time(1));
else
    alltime=data.time;
end

% find correct indices for the samples in alltime/cfg.toi
% to be used later
% look in the time dimension of the data
timeindices=zeros(1,2);
for ii = 1:size(cfg.toi,2)
    [col]=nearest(alltime, cfg.toi(ii));
    timeindices(ii)=col;
end



fprintf(' - ok')

%% define channels, ACT and trials
% ------------------------------------------------------------------------
% select channels
% MW: Note;
% OUTPUT:   channelcombi      = nx2 matrix of indices of the channel
%                               combinations.
%           channelcombilabel = nx2 cell array of labels  of the channel
%                               combinations.
[channelcombi,channelcombilabel] = TEchannelselect(cfg, data, channelselect);
TEprepare.cfg.channelcombi = channelcombi;
TEprepare.cfg.channelcombilabel = channelcombilabel;
uc1 = unique(channelcombi); % the indices of the channels that are used in the pairwise computations (either as target or as source)
% % MW: DEBUG:
% disp('the number of used channels is:')
% disp(max(size(uc1)))
% 
ucl = data.label(uc1); % ... and their labels

% % MW: DEBUG:
% disp('their labels are:')
% disp(ucl)

% MW: Note; At this point uc1 contains the all channelsindices that should
% ever be needed and TEprepare.cfg.channelcombi contains the indeex
% combinations (TEprepare.cfg.channelcombi(:,2) contains the TARGET
% channels) and TEprepare.cfg.channelcombilabel contains the labels

if size(uc1,1) ~= 1
    uc1 = uc1';
end
if size(ucl,1) ~= 1
    ucl = ucl';
end

TEprepare.usedchannels = uc1;
TEprepare.usedchannellabels = ucl;
TEprepare.channelcombi=channelcombi;
TEprepare.channelcombilabel=channelcombilabel;

% % MW: DEBUG: 
% fprintf('\n size of TEprepare.channelcombi:\n')
% disp(size(TEprepare.channelcombi))
% disp('the usedchannel indices are:')
% disp(TEprepare.usedchannels)
% disp('The channelcombinations are:')
% disp(TEprepare.channelcombi)

% calculate ACT matrix (trials x channels) for all (!!) channels in the data,
% not only the used ones (for historical reasons)
% MW: Note; It will be somewhat difficult from here on to access the right
% parts of ACT; Expression should always have the form:
% ACT(uc1(...)), to make sure we only access ACT values of channels we
% actually use.
[ACT]=TEactdetect(datamat,cfg.maxlag,timeindices);
TEprepare.ACT=ACT;

% select trials
% MW: Note; check size of output !
fprintf('\nSelect trials');
[trials,nrtrials]=TEtrialselect(cfg,datamat,ACT,channelcombi);
% % MW: check output of TEtrialselect (once again)
% disp('trails after TEtrialselect')
% disp(trials)
% disp('in total:')
% disp(nrtrials)

TEprepare.trials=trials;
TEprepare.nrtrials=nrtrials;
fprintf(' - ok')

% % calculate max ACT of used trials
% % MW: what is this code good for here? It repeats below ...
% disp('The number of used channels is:');
% disp(TEprepare.usedchannels);

maxact = zeros(1,length(TEprepare.usedchannels));
indexcount = 1;
for ii = TEprepare.usedchannels
    if channelselect==2 % find the right entry in nrtrials below
        % nrtrials only runs is in its first dimension only as long sgncmb
        % we have to identify the fisrt sgncmb that contains
        % TEprepare.usedchannels(indexcount) as its target
        % we do this by comparing usedchannels (in the form of ii) and the channelcombis
        % disp('searching a channelcombi with matching target channel');
        for cci=1:size(TEprepare.channelcombi,1) % loop over channelcombis
            if TEprepare.channelcombi(cci,2)==ii;
                NrTrlIdx=cci; % any channelcombi with the right target will do
            else
               % warning('index of usedchannel not found as target in any combination!')
            end
        end
    elseif channelselect==1
        NrTrlIdx=indexcount;
    else
        error('channelselect undefined in call to TEprepare')
    end
    
    try
    maxact(indexcount) = max(ACT(trials(NrTrlIdx,1:nrtrials(NrTrlIdx)),ii)); % nrtrials should be accessed at the index of the sgncmb that contains the current channel as target
    catch ME
        maxact(indexcount) = NaN; % at least leave a trace of this failure
    end
    indexcount = indexcount+1;
end
TEprepare.maxact = max(maxact); % maxact is indexed as 1: number(usedchannels)
% hence, when accessing maxact in loops do
% not use uc1(...) ! 
clear indexcount

% convert u value from ms to sampling points
if strcmp(cfg.TEcalctype,'V') | strcmp(cfg.TEcalctype,'VW')
    dimu=round(cfg.predicttime_u/1000*data.fsample);
end

%% optimize embedding parameters
% -------------------------------------------------------------------------


% Ragwitz criterion
% ------------------
if strcmp(cfg.optimizemethod, 'ragwitz') == 1
    
    fprintf('\nCalculate optimization with Ragwitz criterion');
    
    % calculate number of trials for all channels
    % this variable cfg.trialselecttype is not specified by the user, it switches
    % the behaviour of TEtrialselect to treat either each channel or just the target
    % channels of an interaction
    cfg.trialselecttype = 1;
    % MW: optrials and optntrials have the size of uc1 here as
    % TEtrialselect is called with this input
    [opttrials,optnrtrials]=TEtrialselect(cfg,datamat,ACT,uc1);
    
    % define max ACT of used trials
    maxact = zeros(1,length(TEprepare.usedchannels));
    indexcount = 1;
    for ii = TEprepare.usedchannels
            if channelselect==2 % find the right entry in nrtrials below
        % nrtrials only runs is in its first dimension only as long sgncmb
        % we have to identify the fisrt sgncmb that contains
        % TEprepare.usedchannels(indexcount) as its target
        % we do this by comparing usedchannels (in the form of ii) and the channelcombis
        % disp('searching a channelcombi with matching target channel');
        for cci=1:size(TEprepare.channelcombi,1) % loop over channelcombis
            if TEprepare.channelcombi(cci,2)==ii;
                NrTrlIdx=cci; % any channelcombi with the right target will do
            else
               % warning('index of usedchannel not found as target in any combination!')
            end
        end
    elseif channelselect==1
        NrTrlIdx=indexcount;
    else
        error('channelselect undefined in call to TEprepare')
    end
    
    try
    maxact(indexcount) = max(ACT(trials(NrTrlIdx,1:nrtrials(NrTrlIdx)),ii)); % nrtrials should be accessed at the index of the sgncmb that contains the current channel as target
    catch ME
        maxact(indexcount) = NaN; % at least leave a trace of this failure
    end
    indexcount = indexcount+1;
    end
    clear indexcount
    
    % define max tau in samples
    maxtau = ceil( min( [max(maxact) cfg.actthrvalue] ) * max(cfg.ragtau) );
    
    %MW debug:
    disp('*****')        
    disp('max(maxact)')
            disp(max(maxact))
            disp('maxtau')
            disp(maxtau)
            disp('cfg.actthrvalue')
            disp(cfg.actthrvalue)
            disp('cfg.repPred')
            disp(cfg.repPred)
            disp('cfg.ragdim')
            disp(cfg.ragdim)
            disp('size(datamat,3) -  min( [max(maxact) cfg.actthrvalue]) - (max(cfg.ragdim)-1)*maxtau - dimu ')
            disp(size(datamat,3) -   min( [max(maxact) cfg.actthrvalue] )- (max(cfg.ragdim)-1)*maxtau - dimu)
    
    
    if strcmp(cfg.TheilerT, 'ACT');
        if cfg.repPred >= size(datamat,3) -  min( [max(maxact) cfg.actthrvalue]) - (max(cfg.ragdim)-1)*maxtau - dimu 
            error(sprintf('TRENTOOL ERROR:\nNot enough points in timeseries for current analysis settings: cfg.repPred too big, or max(cfg.ragdim)*max(cfg.ragtaurange) too big'))
        end
    else
        if cfg.repPred >= size(datamat,3) - cfg.TheilerT - (max(cfg.ragdim)-1)*maxtau - dimu 
            error(sprintf('TRENTOOL ERROR:\nNot enough points in timeseries for current analysis settings: cfg.repPred too big, or max(cfg.ragdim)*max(cfg.ragtaurange) too big'))
        end
    end
    
    datasize4embedding = (ceil(max(cfg.ragtau) * min( [max(maxact) cfg.actthrvalue] ) ) * (max(cfg.ragdim)-1))+1; % the maximum datasize for embedding
    fprintf(['\nData size for embedding: ',num2str(datasize4embedding)]);
    
%     This origin of this condition is unknown 
%     if datasize4embedding > cfg.repPred
%         error('TRANSFER ENTROPY ERROR: max(cfg.ragdim) and/or max(cfg.ragtau) are to high or cfg.repPred to small.')
%     else
%         fprintf(' - ok\n')
%     end
    
    
    
    
    %     % Change to directory containing mex files for the nearest neighbors search
    %     [dir_mex] = TEarch(cfg);
    %     cd(dir_mex);
    
    optdim = nan(length(uc1),max(optnrtrials));
    opttau = nan(length(uc1),max(optnrtrials));
    opttau_act = nan(length(uc1),max(optnrtrials));
    
    % create vector of time points of interest (toi)
    toi=timeindices(1):timeindices(2);
    %     T=length(toi);
    
    
   
    
    
    fprintf('\n')
    for ii=1:length(uc1)
        fprintf('-')
    end
    fprintf('\n')
    
    for channel = 1:length(uc1) % loop over used channels
        fprintf('-')
        for nt = 1:optnrtrials(channel) % loop over trials
            
            % define trainingpoints for Ragwitz criteria depending on
            % TheilerT
            if strcmp(cfg.TheilerT, 'ACT');
                TheilerT=ACT(opttrials(channel,nt),uc1(channel));
            else
                TheilerT=cfg.TheilerT;
            end
            
            % get trial data from data matrix
            dat=squeeze(datamat(opttrials(channel,nt),uc1(channel),toi));
            
            %create emtpy result matrix with nans
            mre = nan(length(cfg.ragdim),length(cfg.ragtau));
            
            
            % loop over ragdim and ragtau
            for rd = 1:length(cfg.ragdim)
                for rt = 1:length(cfg.ragtau)
                    % tau from multiples of ACT to samples
                    tau_sample = ceil(ACT(opttrials(channel,nt),uc1(channel)) * cfg.ragtau(rt));
                    
                    % MW: debug - sanity check for inputs
                    if tau_sample > maxtau
                        fprintf('maxtau %f \n',maxtau)
                        fprintf('tau_sample %f \n', tau_sample)
                        
                        
                        error('TRENTOOL CODE ERROR: tau_sample > maxtau !!!!')
                    end
%                     
                    % calculate Ragwitz - different prediction times have
                    % to be used here depending on the use of the 'V'icente
                    % or the 'V'icente-'W'ibral estimator
                    if strcmp( cfg.TEcalctype ,'V') % use u as the prediction time
                        [mre(rd,rt)] = TEragwitz(cfg,dat,cfg.repPred,dimu,cfg.flagNei,cfg.sizeNei,cfg.ragdim(rd),tau_sample,TheilerT,max(cfg.ragdim),maxtau);
                    elseif strcmp( cfg.TEcalctype ,'VW') % use tau as the prediction time (and u as the interaction delay in TECvalue)
                        [mre(rd,rt)] = TEragwitz(cfg,dat,cfg.repPred,tau_sample,cfg.flagNei,cfg.sizeNei,cfg.ragdim(rd),tau_sample,TheilerT,max(cfg.ragdim),maxtau);
                    else
                        error('Unsupported option in cfg.TEcalctype')
                    end
                end
            end
            
            Smin=eye(size(cfg.ragtau,2)-1);
            S=eye(size(cfg.ragtau,2));
            S(2:end,1:end-1)=S(2:end,1:end-1)+Smin;
            S(1:end-1,2:end)=S(1:end-1,2:end)+Smin;
            S(3,1)=1; S(end-2,end)=1;
            Smoothedmre = (mre*S)./3;
            
            [optdimidx, opttauidx]= find(Smoothedmre == min(min(Smoothedmre)));
            optdim(channel,nt) = cfg.ragdim(min(optdimidx));
            opttau(channel,nt) = cfg.ragtau(min(opttauidx));
            
        end
    end
    
    % tau from samples to multiples of ACT
    for channel = 1:length(uc1)
        for nt = 1:optnrtrials(channel)
            taubyact = opttau(channel,nt) ./ ACT(nt,channel);
            [tauindex]=nearest(cfg.ragtau, taubyact);
            opttau_act(channel,nt) =  cfg.ragtau(tauindex);
        end
    end
    
    
    TEprepare.opttaumat = opttau_act;
    [chanidx, trialidx] = find(opttau_act == max(max(opttau_act)));
    opttaumultiplier = opttau_act(min(chanidx), min(trialidx));
    TEprepare.opttau = opttaumultiplier;
    
    fprintf(strcat(['\nOptimal tau for this dataset may be: ', num2str(opttaumultiplier),'\n']))
    
    % find max dimension
    TEprepare.optdimmat = max(optdim,3);
    TEprepare.optdim = max(max(max(optdim)));
    fprintf(strcat(['\nOptimal dimension for this dataset may be: ', num2str(max(max(optdim))),'\n']))
    
    
    
    
    % Cao criterion
    % --------------
elseif strcmp(cfg.optimizemethod, 'cao') == 1
    
    fprintf('\nCalculate optimization with cao criteria');
    
    % calculate number of trials for all channels
    cfg.trialselecttype = 1;
    [caotrials,caonrtrials]=TEtrialselect(cfg,datamat,ACT,uc1);
    
    
    % Change to directory containing mex files for the nearest neighbors search
    [dir_mex] = TEarch(cfg)
    cd(dir_mex);
    
    
    % scan dimensions for each channel and trial
    % -------------------------------------------------------------------------
    
    maxdim = max(cfg.caodim)+1;
    TEprepare.nrreferencepoints=zeros(length(uc1),nrtrials);
    %     optdim=zeros(length(uc1),maxdim-1);
    
    
    ch_count = 0;
    for channel = 1:length(uc1) %
        ch_count = ch_count +1;
        nt_count = 0;
        for nt = 1:caonrtrials(channel) % MW: caonrtrials has a length equal to length of uc1
            nt_count = nt_count +1;
            
            dat = squeeze(datamat(nt,uc1(channel),timeindices(1)+1:timeindices(2)));
            tau = ceil(cfg.tau*squeeze(ACT(caotrials(nt),uc1(channel))));
            
            % embedding
            T = length(dat);     % length of full time series
            M = T-(maxdim-1)*tau; % number of points inside the time series ready for delay embedding
            query = (1:M);
            
            if M < 1
                error(strcat(['TRANSFER ENTROPY ERROR: maximum caodim (',num2str(max(cfg.caodim)),') is to high for channel ',ucl{uc1(channel)}]))
            end
            
            caodimadded = [cfg.caodim ,max(cfg.caodim)+1];
            
            pointset = zeros(M,maxdim);
            for ii = 1:M
                for jj = caodimadded
                    pointset(ii,jj) = dat(ii+(maxdim-1)*tau-(jj-1)*tau);
                end
            end
            TEprepare.nrreferencepoints(uc1(channel),nt)=size(pointset,1); % MW:. changed channel -> uc1(channel) here
            
            % calculate cao
            [E,Ex]=cao(pointset,query,cfg.caokth_neighbors);
            
            % calculate E1 and E2
            E1(nt_count,ch_count,:)=E(2:end)./E(1:end-1);
            E2(nt_count,ch_count,:)=Ex(2:end)./Ex(1:end-1);
            
            % change because cao !!!!!!!!!!!!!!!!!
            %E1der2=zeros(1,maxdim-2);
            %for kk = 2:maxdim-2
            E1der2=zeros(1,maxdim-3);
            for kk = 2:maxdim-3
                E1der2(kk) = E1(nt_count,ch_count,kk-1)+E1(nt_count,ch_count,kk+1)-2*E1(nt_count,ch_count,kk);
                
            end
            optdim(channel,nt)=find(E1der2==min(E1der2), 1 );
        end
    end
    
    % find max dimension
    TEprepare.optdimmat = optdim;
    TEprepare.optdim = max(max(optdim));
    
    if max(max(optdim)) == max(cfg.caodim)
        error(strcat('TRANSFER ENTROPY ERROR: Optimal dimension found: ', num2str(max(max(optdim))),' is the highest in cfg.caodim. Rerun the data with higher values for cfg.caodim!\n'))
    else
        fprintf(strcat(['Optimal dimension for this dataset may be: ', num2str(max(max(optdim))),'\n']))
    end
    
    TEprepare.cao.E1=E1;
    TEprepare.cao.E2=E2;
    
end

cfg = rmfield(cfg, 'trialselecttype');

cd(working_directory)

fprintf(' - ok\n');



%% add TEprepare structure to the data
% -------------------------------------------------------------------------
fprintf('\nadd TEprepare structure to original data structure ')

TEprepare.cfg=cfg;
varargin{2}.TEprepare = TEprepare;
DataOut = varargin{2};

fprintf(' - ok\ndone\n');

end