function TEpermtest=TEsurrogatestats(varargin)

% TESURROGATESTATS: This function calculates the transfer entropy values
% and performs a permutation test on two transfer entropy data sets (one
% original and one surrogate).
%
% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% !!!      The function TEprepare has to be run on the data first!      !!!
% !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
%
% You can call this function directly as follows:
%         TEsurrogatestats(cfg, data)
%
%
% * REFERENCE INFORMATION
%
%   - transfer entropy
%     - The concept of TE appears in Schreiber's article,
%       "Measuring Information Transfer", Phys. Rev. Lett. 85, 461 - 464
%       (2000).
%     - For the estimation of probability densities needed for the TE
%       computation, the function implements the Kraskov-Stoegbauer-
%       Grassberger estimator described in Kraskov et al. "Estimating
%       mutual information", Phys. Rev. E 69 (6) 066138, (2004).
%
%   - permutation test
%       - Maris & Oostenveld (2007). Nonparametric statistical testing of
%         EEG- and MEG-data. J. of Neuroscience Methods, 164, 177-190.
%
%
% * DEPENDENCIES
%     - Package TSTOOL is used at nearest neighbors searches
%       required for the KSG estimator. (Gnu Public License)
%       http://www.dpi.physik.uni-goettingen.de/tstool/
%     - The following Matlab toolboxes:
%         - signal processing toolbox
%         - statistic toolbox
%     - The functions
%         - TEprepare
%         - transferentropy
%         - TEactdetect
%         - TEchannelselect
%         - TEcmc
%         - TEperm
%         - TEtrialselect
%         - TEvalues
%         - TEwait
%
% * INPUT PARAMETERS
%
%   data            = Fieldtrip raw data structure - it MUST contain:
%       .trials     = cell array (1xnr of trials) containing the data for
%                     each trial
%       .time       = cell (1xnr of trials) containing the time indices for
%                     each trial
%       .label      = cell (1xnr of channels), containing the labels of 
%                     channels included in the data
%       .fsample    = value of sampling rate (in Hertz)
%       .TEprepare  = structure added by TEprepare 
%
% AND
%
%   cfg: The configuration MUST contain:
%
%   cfg.dim         = Number of embedding dimensions. If not specified, the
%                     optimal dimension found in TEprepare will be used, 
%                     which is the recommended option!
%   cfg.tau         = embedding delay in units of act (x*act). If not
%                     specified (recommended option), the tau is used as
%                     followed:
%                     Depending optimizemethod in TEprepare:
%                           'ragwitz' = optimal tau found via ragwitz 
%                                       critrion
%                           'cao'     = cfg.tau given by user in TEprepare
%               
%   cfg.alpha       = significance level for statisatical permutation test 
%                     and correction for multiple comparison 
%                     (default = 0.05)
%   cfg.surrogatetype = 'trialshuffling','trialreverse','blockresampling',
%                     'blockreverse1','blockreverse2', or 'blockreverse3'
%                     surrogate data for trial(n) will be created as
%                     following:
%                     trialshuffling:   trial(n+1)
%                     trialreverse:     reverse of trial(n)
%                     blockresampling:  cuts trial(n) at random point and
%                                       resamples the trial
%                     blockreverse1:    reverse after blockresampling
%                     blockreverse2:    reverse first block after
%                                       blockresampling
%                     blockreverse3:    reverse second block after
%                                       blockresampling
%
%                       examples:
%                       original trial:     1 2 3 4 5
%                       trialshufling:      2 3 4 5 1
%                       trialreverse:       5 4 3 2 1
%                       blockresampling:    3 4 5 1 2
%                       blockreverse1:      2 1 5 4 3
%                       blockreverse2:      5 4 3 1 2
%                       blockreverse3:      3 4 5 2 1
%   cfg.shifttest   = perform shift test to identify instantaneous mixing
%                     between the signal pairs. Values: 'yes' or 'no'
%                     (default = 'yes')
%                     This shift test is important for EEG and MEG data,
%                     because linear mixing is always present in the data.
%                     In case of instantaneous mixing transfer entropy
%                     should not be calculated for the affected
%                     channelpairs with the corresponding parameter sets,
%                     because it could result in false positive results.
%                     Hence the TE values for these cases will be set to
%                     NaN and the corresponding p-values of the permutation
%                     test to 1.
%   cfg.shifttesttype = The shift test can be calculated for the direction
%                     TE value of original data > TE values of shifted data
%                     (value = 'TE>TEshift') or for the other direction
%                     (value = 'TEshift>TE'). In this case the alpha is
%                     set to 0.1 . (default = 'TE>TEshift')
%   cfg.shifttype     = Shifting the data 'onesample' or the length of the 
%                     'predicttime' (default = 'predicttime')
%   cfg.numpermutation = nr of permutations in permutation test
%                     (default = 190100)
%   cfg.permstatstype  = 'mean' to use the distribution of the mean
%                     differences and 'depsamplesT' or
%                     'indepsamplesT' for distribution of the
%                     t-values. (default = 'indepsamplesT')
%   cfg.tail        = 1 tail or 2 tailed test of significance (for the
%                     permutation tests) (default = 2)
%   cfg.correctm    = correction method used for correction of the multiple
%                     comparison problem - False discovery rate 'FDR' or
%                     Bonferroni correction 'BONF' (default = 'FDR')
%   cfg.fileidout   = string for the first part of the output filename.
%
%
%
% * OUTPUT PARAMETERS
%
%
%  TEpermtest
%            .TEpermvalues  = matrix with size:
%                             (channelpair,u,value)
%                           The last dimension "value" includes:
%                           1 - p_values of the statistic within the
%                               distribution given by the permutations
%                           2 - 1 (0), if the statistics is significant at
%                               the prescribed alpha level (or not)
%                           3 - 1 (0), if the statistics is significant
%                               after correction for mulitple comparisons
%                               (or not)
%                           4 - 1 (0), mean difference or tvalue of mean
%                               difference depending on cfg.permstatstype
%                           5 - 1 (0), if instantaneous mixing (volume
%                               conduction) exists (or not)
%            .dimord        = dimensions of TEpermvalues
%            .cfg           = configuration file used to calculate TE and
%                             permtest
%            .sgncmb        = labels of channel combinations (source ->
%                             target)
%            .numpermutation = number of permutations
%            .ACT           = structure including
%                .act       = ACT matrix (trial x channel)
%                .label     = label of channels in ACT matrix
%            .TEprepare     = results of the function TEprepare fron the
%                             data
%
% AND 
%
%  TEresult             = Output structure of the function tranferentropy
%          .TEmat       = resultmatrix including transfer entropy(TE)
%                         values (channelpairs x u x trial)
%          .MImat       = resultmatrix including mutual information (MI)
%                         values (channelpairs x u x trial)
%          .dimord      = 'channelpair_u_trial'; the dimensions of TEmat 
%                         and MImat
%          .cfg         = configuration file used to calculate TE
%          .trials      = trial numbers selected from raw dataset
%          .act         = ACT matrix (trial x (target) channel)
%          .sgncmb      = labels of channel combinations (source -> target)
%          .TEprepare   = results of the function TEprepare from the
%                         data
%   if instantaneous mixing is found in the data, then another field will 
%   be added:
%          .instantaneousmixing = matrix (channel x u) which indicates were
%          the instantaneous mixings were found (1) or not (0).%
%
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation;
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY;
%
% Version 1.0 by Michael Lindner, Raul Vicente, Michael Wibral
% Frankfurt 2009
%

%% Remember the working directory
working_directory1 = pwd;

%% check data
% -------------------------------------------------------------------------
fprintf('\nCheck data and config');

% check the input alternative
if isfield(varargin{1},'surrogatetype') && isstruct(varargin{1}) && isstruct(varargin{2}) && isfield(varargin{2},'trial')
    cfg =  varargin{1};
    data = varargin{2};
else
    error('\nTRANSFER ENTROPY ERROR: incorrect input values, see help!');
end

% check if TEprepare was performed
if ~isfield(data, 'TEprepare'),
    error('\nTRANSFER ENTROPY ERROR: The function TEprepare must be performed on the data, see help!');
end;

% check data using checkdata from Fieldtrip
[data] = checkdata(data, 'datatype','raw');

% check the data structure
if ~isfield(data, 'trial'),
    error('\nTRANSFER ENTROPY ERROR: data must be in ''.trial''-structure, see help!');
end;
if ~isfield(data, 'time'),
    error('\nTRANSFER ENTROPY ERROR: data contains no ''.time''-structure, see help!');
end;
if ~isfield(data, 'label'),
    error('\nTRANSFER ENTROPY ERROR: data contains no ''.label''-structure, see help!');
end;
if ~isfield(data, 'fsample'),
    error('\nTRANSFER ENTROPY ERROR: data contains no ''.fsample''-structure, see help!');
end;
if size(data.time,1)>size(data.time,2)
    data.time=data.time';
end

% compare new cfg and cfg from TEprepare if equal fields exist
% -------------------------------------------------------------------------

doublefields = 0;
cfgTEprepare = varargin{2}.TEprepare.cfg;
if isfield(cfg, 'Path2TSTOOL') && isfield(cfgTEprepare, 'Path2TSTOOL')
    cfgTEprepare = rmfield(cfgTEprepare, 'Path2TSTOOL');
end

cfgfields = fieldnames(cfgTEprepare);
cfgfields2 = fieldnames(cfg);

for ii = 1:size(cfgfields,1);
    for jj = 1:size(cfgfields2,1);
        if strcmp(cfgfields{ii},cfgfields2{jj})
            doublefields = doublefields + 1;
        end
    end
end


if doublefields  > 0
    fprintf('\n')
    error('\nTRANSFER ENTROPY ERROR: Illegal attempt to overwrite entry generated by or used for TEprepare! Change cfg or rerun TEprepare. (see help)')
end


% add structures and values of data.TEprepare.cfg to cfg
names1 = fieldnames(data.TEprepare.cfg);
nr1 = size(names1,1);
for ii = 1:nr1
    eval(strcat('cfg.',names1{ii},' = getfield(data.TEprepare.cfg, {1}, names1{ii});'))
end


% check configuration and set defaults
% -------------------------------------------------------------------------

% if not defined set defaults
if ~isfield(cfg, 'alpha'),          cfg.alpha = 0.05;           end;
if ~isfield(cfg, 'correctm'),       cfg.correctm = 'FDR';       end;
if ~isfield(cfg, 'tail'),           cfg.tail = 2;               end;

if ~isfield(cfg, 'permstatstype'),  cfg.permstatstype = 'mean'; end;
if strcmp(cfg.permstatstype , 'mean') == 0 && strcmp(cfg.permstatstype , 'indepsamplesT') == 0 && strcmp(cfg.permstatstype , 'depsamplesT') == 0
    error('\nTRANSFER ENTROPY ERROR: wrong cfg.permstatstype - use ''mean'' ''depsamplesT'' or ''indepsamplesT'', see help!');
end

if ~isfield(cfg, 'shifttest'),  cfg.shifttest = 'yes'; end;
if strcmp(cfg.shifttest , 'yes') == 0 && strcmp(cfg.shifttest , 'no') == 0
    error('\nTRANSFER ENTROPY ERROR: wrong cfg.shifttest - use ''yes'' or ''no'', see help!');
end
if strcmp(cfg.shifttest , 'yes') 
    if ~isfield(cfg, 'shifttype'),    cfg.shifttype = 'predicttime'; end;
    if ~isfield(cfg, 'shifttesttype'),  cfg.shifttesttype = 'TE>TEshift'; end;
    if strcmp(cfg.shifttesttype , 'TE>TEshift') == 0 && strcmp(cfg.shifttesttype , 'TEshift>TE') == 0
        error('\nTRANSFER ENTROPY ERROR: wrong cfg.shifttesttype - use ''TE>TEshift'' or ''TEshift>TE'', see help!');
    end
end
    
if ~isfield(cfg, 'fileidout'),
    error('\nTRANSFER ENTROPY ERROR: cfg.fileidout must be defined, see help!');
end;

%%%%%%%%%%%%%%%%%
% check TE parameter

if ~isfield(cfg, 'dim'), cfg.dim = data.TEprepare.optdim;  end;
if cfg.dim < data.TEprepare.optdim
    fprintf('\nTRANSFER ENTROPY WARNING: specified embedding dimension (cfg.dim) is smaller then the optimal dimension from TEprepare.')
elseif cfg.dim > data.TEprepare.optdim
    fprintf('\nTRANSFER ENTROPY WARNING: specified embedding dimension (cfg.dim) is bigger then the optimal dimension from TEprepare.')
end



if strcmp(data.TEprepare.cfg.optimizemethod, 'ragwitz') 
    cfg.tau = data.TEprepare.opttau;
elseif strcmp(data.TEprepare.cfg.optimizemethod, 'cao') 
    cfg.tau = data.TEprepare.cfg.tau;
end


if isempty(cfg.predicttime_u), error('\nTRANSFER ENTROPY ERROR: specify cfg.predicttime_u, see help!');  end;

if ~isfield(cfg, 'kth_neighbors'),  cfg.kth_neighbors = 4;  end;

if ~isfield(cfg, 'TheilerT'),       cfg.TheilerT = 'ACT';   end;
if ~strcmp(cfg.TheilerT, 'ACT');
    if size(cfg.TheilerT,1)>1 || size(cfg.TheilerT,2)>1
        error('\nTRANSFER ENTROPY ERROR: cfg.TheilerT must include a scalar, see help!');
    end
end
%%%%%%%%%%%%%%%%%

% check the format of input vectors
if size(cfg.toi,1)>size(cfg.toi,2)
    cfg.toi=cfg.toi';
elseif size(cfg.dim,1)>1 || size(cfg.dim,2)>1
    error('\nTRANSFER ENTROPY ERROR: cfg.dim must include a scalar, see help!');
elseif size(cfg.predicttime_u,1)>size(cfg.predicttime_u,2)
    cfg.predicttime_u=cfg.predicttime_u';
elseif size(cfg.kth_neighbors,1)>1 || size(cfg.kth_neighbors,2)>1
    error('\nTRANSFER ENTROPY ERROR: cfg.dim must include a scalar, see help!');
end



fprintf(' - ok');


%% read data
% -------------------------------------------------------------------------
fprintf('\nRead data');

datamat=zeros(size(data.trial,2),size(data.trial{1},1),size(data.trial{1},2));
for ii = 1:size(data.trial,2)
    datamat(ii,:,:)=data.trial{ii};
end


% read time values of the data
% use rounding for later use of the 'find' command
if iscell(data.time)
    alltime=cell2mat(data.time(1));
else
    alltime=data.time;
end

% find correct indices for the samples in alltime/cfg.toi
% to be used later
% look in the time dimension of the data
timeindices=zeros(1,2);
for ii = 1:size(cfg.toi,2)
    [col]=find(abs(alltime - cfg.toi(ii))==min(abs(alltime - cfg.toi(ii))));
    timeindices(ii)=col;
end

fprintf(' - ok');



%% get channels, ACT and trials from the cfg.TEprepare
% ------------------------------------------------------------------------

% select channels
%[channelcombi,channelcombilabel] = TEchannelselect(cfg, data, channelselect);
channelcombi=data.TEprepare.channelcombi ;
%channelcombilabel=data.TEprepare.channelcombilabel ;
cfg.permtest.channelcombi = channelcombi;
cfg.permtest.channelcombilabel = data.TEprepare.channelcombilabel ;

% % calculate ACT matrix (trials x channels)
% [ACT]=TEactdetect(datamat,cfg.maxlag,timeindices);
ACT=data.TEprepare.ACT;
cfg.permtest.ACT=ACT;

% select trials
% [trials,nrtrials]=TEtrialselect(cfg,datamat,ACT,channelcombi);
trials=data.TEprepare.trials;
nrtrials=data.TEprepare.nrtrials;
cfg.permtest.trials=trials;
cfg.permtest.nrtrials=nrtrials;

%% check nr of permutations
% -------------------------------------------------------------------------
fprintf('\n\nChecking number of permutations\n');

nr2cmc=size(channelcombi,1)*size(cfg.predicttime_u,2);

if ~isfield(cfg, 'numpermutation'),
    cfg.numpermutation = 190100; % for p<0.01 with a possible bonferroni correcetion of 100
elseif cfg.numpermutation < ceil(1/cfg.alpha)
    error(strcat('\nTRANSFER ENTROPY ERROR: cfg.numpermutation too small!'));
else
    if nrtrials>31
        if cfg.numpermutation > 2^31
            error(strcat('\nTRANSFER ENTROPY ERROR: cfg.numpermutation too huge!'));
        end
    else
        if cfg.numpermutation > 2^min(nrtrials)
            error(strcat('\nTRANSFER ENTROPY ERROR: cfg.numpermutation too huge!'));
        end
    end
    if cfg.numpermutation < ceil(1/(cfg.alpha/nr2cmc))
       fprintf('\n#######################################################################################\n# WARNING: Nr of permutations not sufficient for correction for multiple comparisons! #\n#######################################################################################\n'); 
    end
end

fprintf(' - ok\n');


%% start calculating TE
% -------------------------------------------------------------------------

cfg.calctime = 'yes';

% for unshuffled data
% ----------------------
fprintf('\nStart calculating transfer entropy for unshuffled data');
cfg.shuffle = 'no';
[TEresult] = transferentropy(cfg,data);
TEresult.TEprepare = data.TEprepare;

% %$ML
% save(strcat(cfg.fileidout,'_TEresultorig'), 'TEresult','-v7.3');

cfg.calctime = 'no';

% for shifted data
% ----------------------
% TEshift is created inside transferentropy.m as a reduced version of
% TEresult without certain fields. TEshift is never written to disk/file
% to avoid later confusion. Please save TEshift yourself if necessary.
if strcmp(cfg.shifttest, 'yes')
    fprintf('\nStart calculating transfer entropy for shifted data');
    cfg.shuffle = 'no';
    [TEshift] = transferentropy(cfg,data,'shifttest');

%     %$ML
%     save(strcat(cfg.fileidout,'_TEshift'), 'TEshift','-v7.3');

    
    % permutation test for shift test
    fprintf('\nStart permutation tests for shift test');
    permstatstype = cfg.permstatstype;
    cfg.permstatstype = 'indepsamplesT';
    tailtype = cfg.tail;
    cfg.tail = 1;
    if strcmp(cfg.shifttesttype, 'TE>TEshift')
        alpha = cfg.alpha;
        cfg.alpha = 0.05;
        TEpermshift = TEperm(cfg,TEresult,TEshift);
        cfg.alpha = alpha;
    elseif strcmp(cfg.shifttesttype, 'TEshift>TE')
        alpha = cfg.alpha;
        cfg.alpha = 0.1;
        TEpermshift = TEperm(cfg,TEshift,TEresult);
        cfg.alpha = alpha;
    end
    cfg.permstatstype = permstatstype;
    cfg.tail=tailtype;
    
    
    
%     %$ML
%     save(strcat(cfg.fileidout,'_TEpermshift'), 'TEpermshift','-v7.3');

    
    % analyze shift test
    fprintf('\nanalyze shift test\n');
    
    % MW: check if there are NaNs in TEresult from poor initialization
    NaNidx=find(isnan(TEresult.TEmat));
    if size(NaNidx,1) ~= 0
        disp('Found NaN in TEresult.TEmat! Aborting')
        return
    end
    
    
    if strcmp(cfg.shifttesttype, 'TE>TEshift')
        indexinstmix = find(TEpermshift.TEpermvalues(:,:,2)==0);
        if size(indexinstmix,1) == 0
            fprintf('No instantaneous mixing found!\n')
        else
            fprintf(strcat(num2str(size(indexinstmix,1)),' instantaneous mixings found by strict shifttest!\nFor these cases TEvalues of all trials are set to NaN!\n'))
            mask=repmat((TEpermshift.TEpermvalues(:,:,2)-1)*-1, [1 1 size(TEresult.TEmat,3)]);
            TEresult.TEmat(mask==1) = NaN;
            TEresult.MImat(mask==1) = NaN;
            clear mask;
            TEresult.instantaneousmixing = (TEpermshift.TEpermvalues(:,:,2)-1)*-1;
        end
    elseif strcmp(cfg.shifttesttype, 'TEshift>TE')
        indexinstmix = find(TEpermshift.TEpermvalues(:,:,2)==1);
        if size(indexinstmix,1) == 0
            fprintf('No instantaneous mixing found!\n')
        else
            fprintf(strcat(num2str(size(indexinstmix,1)),' instantaneous mixings found by non-strict shifttest!\nFor these cases TEvalues of all trials are set to NaN!\n'))
            mask=repmat(TEpermshift.TEpermvalues(:,:,2), [1 1 size(TEresult.TEmat,3)]);
            TEresult.TEmat(mask==1) = NaN;
            TEresult.MImat(mask==1) = NaN;
            clear mask;
            TEresult.instantaneousmixing = TEpermshift.TEpermvalues(:,:,2);
        end
    end
    
    clear TEpermshift
end

% for shuffled data
% ----------------------
% TEshuffle is created inside transferentropy.m as a reduced version of
% TEresult without certain fields. TEshuffle is never written to disk/file
% to avoid later confusion. Please save TEshuffle yourself if necessary.
fprintf('\nStart calculating transfer entropy for shuffled data');
cfg.shuffle = 'yes';
[TEshuffle] = transferentropy(cfg,data);

cfg = rmfield(cfg, 'shuffle');
cfg = rmfield(cfg, 'calctime');

%     %$ML
%    save(strcat(cfg.fileidout,'_TEshuffle'), 'TEshuffle','-v7.3');



%% permutation tests
% -------------------------------------------------------------------------
fprintf('\nStart permutation tests');

%TEpermtest=[];

TEpermtest = TEperm(cfg,TEresult,TEshuffle);

TEpermtest.dimord = 'chanpair_u_value';
TEpermtest.cfg = cfg;
TEpermtest.ACT.label = data.label;
TEpermtest.ACT.actvalue = ACT;
TEpermtest.sgncmb = TEresult.sgncmb;
TEpermtest.numpermutation = cfg.numpermutation;
TEpermtest.TEprepare = data.TEprepare;
fprintf('\nCalculation ready\n')


%% save results
% -------------------------------------------------------------------------
fprintf('\nSaving ...')
fprintf('\nResults of TE')
save(strcat(cfg.fileidout,'_time',num2str(cfg.toi(1)),'-',num2str(cfg.toi(2)),'s_TE_output.mat'), 'TEresult','-v7.3');
fprintf(' - ok');
fprintf('\nResults of permutation test')
save(strcat(cfg.fileidout,'_time',num2str(cfg.toi(1)),'-',num2str(cfg.toi(2)),'s_TEpermtest_output.mat'), 'TEpermtest','-v7.3');
fprintf(' - ok');


%% Returning to the working directory
cd(working_directory1)


fprintf('\n\nThank you for using this transfer entropy tool!\n')

return;

