function [TEpermtest] = TEperm(cfg,TEresult1,TEresult2)

% TEPERM: This function performs a permutation test on two sets of data and
% is called from the function transferentropystats (MW: NAME CORRECT
% CALLING FUNCTIONS HERE)
%
% * INPUT PARAMETERS
%
%
%   TEresult1 and TEresult = result structures of the function
%                            transferentropy
%
% AND
%
%   cfg: The configuration MUST contain:
%
%   cfg.alpha          = required significance level
%   cfg.tail           = 1 tail or 2 tail testing; tail 1 = one-tailed test
%                          for TEresult1 > TEresult2
%   cfg.numpermutation = nr of permutations
%   cfg.correctm       = for cmc
%   cfg.permstatstype  = 'mean', 'indepsamplesT' or 'depsamplesT'
%
% * OUTPUT PARAMETERS
%
%  TEpermtest
%            .TEpermvalues  = matrix with size:
%                             (channelpair,dim,tau,u,kth,T,value)
%                           The last dimension "value" includes:
%                           1 - p_values of the statistic within the
%                               distribution given by the permutations
%                           2 - 1 (0), if the statistics is significant at
%                               the prescribed alpha level (or not)
%                           3 - 1 (0), if the statistics is significant
%                               after correction for mulitple comparisons
%                               (or not)
%                           4 - the values of the test statistic, mean
%                               difference or tvalue of mean
%                               difference depending on cfg.permstatstype
%                           5 - 1 (0), if instantaneous mixing (volume
%                               conduction) exists (or not)
%            .cfg           = configuration file used to calculate TE and
%                             permtest
%            .label         = labels of used channels
%            .sgncmb        = labels of channel combinations (source ->
%                             target)
%            .numpermutation = number of permutations
%            .ACT           = structure including
%                .act       = ACT matrix (trial x channel)
%                .label     = label of channels in ACT matrix
%            .TEprepare     = results of the function TEprepare fron the
%                             data
%
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation;
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY;
%
% Version 1.0 by Michael Lindner, Raul Vicente, Michael Wibral
% Frankfurt 2009
%

%% Remember the working directory
working_directory = pwd;

%% set new stream for random numbers
RandStream.setDefaultStream(RandStream('mt19937ar','seed',sum(100*clock)));


fprintf('\nPerform permutation tests');

datax = TEresult1.TEmat;
datay = TEresult2.TEmat;


% Some definitions (extracting the number of trials)
n = size(datax,3);
m = size(datay,3);

% sanity check
if strcmp(cfg.permstatstype, 'depsamplesT')
    if n~=m
        error('TRANSFER ENTROPY ERROR: illegal option size of the two datasets are not equal, see help!');
    end
end

%% Computing the basic statistics
meandatax = mean(datax,3);
meandatay = mean(datay,3);

if strcmp(cfg.permstatstype, 'mean')
    TEstatistic = meandatax-meandatay;
elseif strcmp(cfg.permstatstype, 'indepsamplesT')
    % MW: found typo here: squaring was of the repmat part NOT of the
    % differences -> corrected 2010-04-19
    SoSx= sum((datax-repmat(meandatax,[1 1 size(datax,3)])).^2,3);
    SoSy= sum((datay-repmat(meandatay,[1 1 size(datay,3)])).^2,3);
    DiffSoSxy = (SoSx - SoSy);
    % MW: "sqrt" does takes the elementwise root - this is correct
    S = sqrt( abs( DiffSoSxy ./ (n+m-2) ));
    % MW: took aout the "abs(.)" from this sqaure root because inside can
    % never be negative
    TEstatistic = (((meandatax-meandatay)./S) .* (sqrt(n*m/(n+m))));
    clear SoSx SoSy DiffSoSxy S
elseif strcmp(cfg.permstatstype, 'depsamplesT')
    Diffd = datax-datay;
    Dd = mean((datax-datay),3);
    Sdd = sqrt(var(Diffd,0,3));
    TEstatistic = sqrt(n)*Dd./Sdd;
    clear Diffd Dd Sdd
end



% Start permutation test
% -------------------------------------------------------------------------
% (To achieve a CV of 0.1% at the ALS of 0.05 we need 1901 different
% permutations)

TEpermdist=zeros(size(datax,1),size(datax,2),cfg.numpermutation);

if strcmp(cfg.permstatstype, 'mean')
    dist_x=zeros(size(datax,1),size(datax,2),cfg.numpermutation);
    dist_y=zeros(size(datay,1),size(datay,2),cfg.numpermutation);
end

fprintf('\nGenerate permutations\n');

for kk = 1:20
    fprintf('-')
end
fprintf('\n')

for pp = 1:cfg.numpermutation
    
    %   text waitbar
    if mod(pp,floor(cfg.numpermutation/20))==0
        fprintf('-')
    end
    
    if strcmp(cfg.permstatstype, 'mean')

        % Pool the data
        data_pool = cat(3, datax, datay);

        % Permuting indexes from 1 to n+m
        [ignore,permu] = sort(rand(1,m+n));
        kkt=permu(1:n);
        data4test_x=data_pool(:,:,kkt);
        kkt2=permu(n+1:m+n);
        data4test_y=data_pool(:,:,kkt2);

        dist_x(:,:,pp) = mean(data4test_x,3);
        dist_y(:,:,pp) = mean(data4test_y,3);
        
        
    elseif strcmp(cfg.permstatstype, 'indepsamplesT')

        % Pool the data
        data_pool = cat(3, datax, datay);

        % Permuting indexes from 1 to n+m
        [ignore,permu] = sort(rand(1,m+n));
        kkt=permu(1:n);
        data4test_x=data_pool(:,:,kkt);
        kkt2=permu(n+1:m+n);
        data4test_y=data_pool(:,:,kkt2);
        meandata4test_x = mean(data4test_x,3);
        meandata4test_y = mean(data4test_y,3);

        % MW: found typo here: squaring was of the repmat part NOT of the
        % differences -> corrected 2010-04-19
        pSoSx= sum((data4test_x-repmat(meandata4test_x,[1 1 size(data4test_x,3)])).^2,3);
        pSoSy= sum((data4test_y-repmat(meandata4test_y,[1 1 size(data4test_y,3)])).^2,3);
        % MW: added an abs(.) here because pSoSx < pSoSy is well possible
        Sp = sqrt( abs(pSoSx - pSoSy)./ (n+m-2) );
        Tp = ((meandata4test_x-meandata4test_y)./Sp) .* (sqrt(n*m/(n+m)));
        TEpermdist(:,:,pp) = Tp;

    elseif strcmp(cfg.permstatstype, 'depsamplesT')

        % Permuting pairs

        a = rand(n,1); % UNIFORMLY distributed random numbers between zero and 1
        permvector = zeros(n,1);
        permvector(a<=.50)=-1;
        permvector(a>.50)=1;
        % This would fail if there is only ONE signal combination
        % (cfg.sgncmb) because the last dimension will be singleton and
        % flattened therefore
        % BUT permute (2D-matrix,[3 2 1]) will in this case create a third dimension
        mm = permute(repmat(permvector, [1,size(datax,2),size(datax,1)]),[3 2 1]);
        data4test_x = datax .* mm;%repmat(permvector, [size(datax,1),size(datax,2),size(datax,3)]);
        data4test_y = datay .* mm;%repmat(permvector, [size(datay,1),size(datay,2),size(datay,3)]);

        Diff = data4test_x-data4test_y;
        D = mean((data4test_x-data4test_y),3);
        Sd = sqrt(var(Diff,0,3));
        Tp = sqrt(size(D,1))*D./Sd;
        TEpermdist(:,:,pp) = Tp;
    end

end

if strcmp(cfg.permstatstype, 'mean')
    TEpermdist = dist_x-dist_y;
end


fprintf(' - ok');

% Evaluating the quantiles of the true results in the permdistribution
fprintf('\nStart permutation tests');

z = sort(TEpermdist,3);

TEpermvalues = NaN(size(datax,1),size(datax,2),5); 

% prepare text waitbar
fprintf('\nPlease wait...\n\n');
for ii = 1:size(datax, 1)
    fprintf('-')
end
fprintf('\n')

for channelpair = 1:size(datax,1) % loop over singalcombinations
    fprintf('-');
    for u=1:size(datax,2) % loop over prediction times u

        if isnan(TEstatistic(channelpair,u)) == 1 % check if a preceeding shiftest has indicated inst. mixing
            TEpermvalues(channelpair,u,1) = 1;
            TEpermvalues(channelpair,u,2) = 0;
            TEpermvalues(channelpair,u,3) = 0;
            TEpermvalues(channelpair,u,4) = 0;
            TEpermvalues(channelpair,u,5) = 1; % if a NaN was found indicate this as instantaneous mixing

        else
            % Checking the significance level
            if cfg.tail == 1
                TEpermvalues(channelpair,u,1) = length(find(z(channelpair,u,:)>TEstatistic(channelpair,u)))/cfg.numpermutation;
                if TEpermvalues(channelpair,u,1) == 0 % MW: why 0.0 not 0 ??
                    TEpermvalues(channelpair,u,1) = 1/cfg.numpermutation;
                end
                %TEpermvalues(channelpair,u,2) = 0;
%                 if TEpermvalues(channelpair,u,1) <cfg.alpha
%                     TEpermvalues(channelpair,u,2) = 1;
%                 else
%                     TEpermvalues(channelpair,u,2) = 0;
%                 end


                %%%%%%%%%%%%%%%%%MW: the two-tailed section needed to be fixed %%%%%
            elseif cfg.tail == 2

                % MW: This was the old code %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                %                 if TEstatistic(channelpair,u,1) >= 0
                %                     TEpermvalues(channelpair,u,1) = length(find(z(channelpair,u,:)>TEstatistic(channelpair,u)))/cfg.numpermutation;
                %                     if TEpermvalues(channelpair,u,1) == 0.0
                %                         TEpermvalues(channelpair,u,1) = 1/cfg.numpermutation;
                %                     end
                %                 elseif TEstatistic(channelpair,u,1) < 0
                %                     TEpermvalues(channelpair,u,1) = length(find(z(channelpair,u,:)<TEstatistic(channelpair,u)))/cfg.numpermutation;
                %                     if TEpermvalues(channelpair,u,1) == 0.0
                %                         TEpermvalues(channelpair,u,1) = 1/cfg.numpermutation;
                %                     end
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                %%%%% NEW code 2010-04-19 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                percentilebigger = length(find(z(channelpair,u,:)>TEstatistic(channelpair,u)))/cfg.numpermutation;
                percentilesmaller = length(find(z(channelpair,u,:)<TEstatistic(channelpair,u)))/cfg.numpermutation;
                percentilemoreextreme = min(percentilebigger,percentilesmaller); % take minimum of two POSITIVE numbers!
                TEpermvalues(channelpair,u,1) = percentilemoreextreme;
                if percentilemoreextreme == 0
                    TEpermvalues(channelpair,u,1) = 1/cfg.numpermutation;
                end
                %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            end
            %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            if TEpermvalues(channelpair,u,1) <cfg.alpha/2
                TEpermvalues(channelpair,u,2) = 1;
            else
                TEpermvalues(channelpair,u,2) = 0;
            end
            TEpermvalues(channelpair,u,5) = 0;
        end
        TEpermvalues(channelpair,u,4) = TEstatistic(channelpair,u);
    end
end


fprintf(' - ok');

%% Correction for multiple comparisons
fprintf('\nCorrection for multiple comparison...')
pvalues = TEpermvalues(:,:,1);

% correct only for all passing instantaneous mixing test
if isfield(TEresult1, 'instantaneousmixing')
    instmixing1 = TEresult1.instantaneousmixing;
else
    instmixing1 = zeros(size(pvalues,1),size(pvalues,2));
end
if isfield(TEresult2, 'instantaneousmixing')
    instmixing2 = TEresult2.instantaneousmixing;
else
    instmixing2 = zeros(size(pvalues,1),size(pvalues,2));
end
mixmask = instmixing1 + instmixing2;

% mm0 = find(mixmask==0);

nrinstmix =  size(pvalues,1)*size(pvalues,2) - length(find(mixmask==0));


significance = TEcmc(pvalues, cfg.correctm, cfg.alpha, nrinstmix);
TEpermvalues(:,:,3)=significance;
fprintf(' - ok\n');

%TEpermtest.TEpermdist=TEpermdist;
TEpermtest.TEpermvalues=TEpermvalues;


%% Returning to the working directory
cd(working_directory)

return;

