function [TEresult]=transferentropy(cfg, data, varargin)

%
% TRANSFERENTROPY computes the transfer entropy (TE) among given pairs of
% channels for a sequence of trials over periods of time
%
% This function is called by the function TEsurrogatestats,
% TEconditionstatssingle,TEconditionsstatsgroup, and TEgroupstats
%
% !! varargin is only used for the shift test when called from the function
% TEsurrogatestats, TEconditionstatssingle or TEgroup_calculate !!
%
%
% * REFERENCE INFORMATION
%     - The concept of TE appears in Schreiber's article,
%       "Measuring Information Transfer", Phys. Rev. Lett. 85, 461 - 464
%       (2000).
%     - For the estimation of probability densities needed for the TE
%       computation, the function implements the Kraskov-Stoegbauer-
%       Grassberger estimator described in Kraskov et al. "Estimating
%       mutual information", Phys. Rev. E 69 (6) 066138, (2004).
%
% * DEPENDENCIES
%     - Package TSTOOL is used at nearest neighbors searches
%       required for the KSG estimator. (Gnu Public License)
%       http://www.dpi.physik.uni-goettingen.de/tstool/
%     - The following Matlab toolboxes:
%           - signal processing toolbox
%           - statistic toolbox
%     - The functions
%           - TEactdetect
%           - TEchannelselect % ToDo: change to Fieldtrip's CHANNELSELECT
%           function
%           - TEtrialselect
%           - TEvalues
%           - TECvalues
%
%
% * INPUT PARAPETERS
%
%   data            = Fieldtrip datastructure MUST contain:
%       .trials     = three dimensional data matrix
%       .time       = vector 1 x numtoi, the time included in the
%                     data
%       .label      = vector 1 x numlabel, labels of channelnames included
%                     in the data
%       .fsample    = value of sampling rate
%
%   The configuration MUST contain:
%   cfg.sgncmb      = list of channelpairs
%                     cell array (Nx(source, target))
%   or
%   cfg.channel     = list of channels testing all by all
%
%   and
%
%   cfg.toi         = the time range of interest (vector 1 x numtoi) in ms
%                     e.g. (time_from, time_to)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   WARNING:
%   The span of time needed for embedding is: (max(dim)-1)*max(tau)
%   The prediction time starts after this embedding time. Hence the span of
%   time defined in cfg.toi must be a good deal longer than the embedding
%   time, at least embedding time plus 150 samples or max(cfg.predicttime_u).
%
%       |<  embedding time  >|< prediction time u...
%   ----|--------------------|-----------------------------------|-->
%       |<                       cfg.toi                        >|
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   cfg.dim         = embedding dimension
%   cfg.predicttime_u = points ahead for the advance vector in ms
%                     (vector 1 x numk)
%   cfg.kth_neighbors = number of neighbors for fixed mass search (controls
%                     balance of bias/statistical errors) (default = 4)
%   cfg.TheilerT    = number of temporal neighbors excluded to avoid serial
%                     correlations (Theiler correction) (default = 'ACT')
%   cfg.trialselect = ACT threshholding of trials - 'ACT' ,'range' or 'no'
%                     (default = 'ACT')
%       if you chose 'ACT' or nothing add:
%       cfg.actthrvalue = max threshold for ACT; min threshold default = 0
%       cfg.minnrtrials = minimum Nr of trials with ACT < actthrest used to
%                     calculate transfer entropy
%       if you chose 'range' add:
%       cfg.trial_from  = Inferior limit for the trials to be considered
%       cfg.trial_to    = Superior limit for the trials to be considered
%   cfg.maxlag      = the range of lags for computing the auto correlation
%                     time: from -MAXLAG to MAXLAG (default = 1000)
%   cfg.surrogatetype = 'trialshuffling','trialreverse','blockresampling',
%                     'blockreverse1','blockreverse2', or 'blockreverse3'
%
%
%                       original trial:     1 2 3 4 5
%                       trialreverse:       5 4 3 2 1
%                       blockresampling:    3 4 5 1 2
%                       blockreverse1:      2 1 5 4 3
%                       blockreverse2:      5 4 3 1 2
%                       blockreverse3:      3 4 5 2 1
%
% * OUTPUT PARAMETERS
%
%  TEresult             = Output structure
%          .TEmat       = resultmatrix including transfer entropy(TE)
%                         values
%          .MImat       = resultmatrix including mutual information (MI)
%                         values
%          .dimord      = dimensions of TEmat and MImat
%          .cfg         = configuration file used to calculate TE
%          .trials      = trial numbers selcted from raw dataset
%          .act         = ACT matrix (trial x (target)channel)
%          .sgncmb      = labels of channel combinations (source -> target)
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation;
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY;
%
% Version 1.0 by Raul Vicente, Michael Wibral, ... and Michael Lindner
% Frankfurt 2009
%
% CHNAGELOG:
%
% 2011-08-09:  MW - at each call to TEvalues.m I added an if else statement
% to sewicth to the use of TCvalues if data.TEprepare.cfg.TEcalctype is
% 'VW' - i.e. the use of the new predictor is desired by the user.

%% Remember the working directory
working_directory = pwd;

% check data
% -------------------------------------------------------------------------
fprintf('\nCheck data and config');

[data] = checkdata(data, 'datatype','raw');

% check the data structure
if ~isfield(data, 'trial'),
    error('\nTRANSFER ENTROPY ERROR: data must be in ''.trial''-structure, see help!');
end;
if ~isfield(data, 'time'),
    error('\nTRANSFER ENTROPY ERROR: data contains no ''.time''-structure, see help!');
end;
if ~isfield(data, 'label'),
    error('\nTRANSFER ENTROPY ERROR: data contains no ''.label''-structure, see help!');
end;
if ~isfield(data, 'fsample'),
    error('\nTRANSFER ENTROPY ERROR: data contains no ''.fsample''-structure, see help!');
end;
if size(data.time,1)>size(data.time,2)
    data.time=data.time';
end

% debug notification
if strcmp(data.TEprepare.cfg.TEcalctype,'VW')
    disp('******** transferentropy.m using the VW-type estimator****')
end

% check configuration and set defaults
% -------------------------------------------------------------------------

% if not defined set defaults
if ~isfield(cfg, 'maxlag'),       cfg.maxlag = 1000;                 end;
if ~isfield(cfg, 'trialselect'),  cfg.trialselect = 'ACT';           end;
if ~isfield(cfg, 'shuffle'),      cfg.shuffle = 'no';                end;

% check if channel or channelcombinations are defined
if isfield(cfg, 'channel') && isfield(cfg, 'sgncmb') ,
    error('\nTRANSFER ENTROPY ERROR: specify cfg.channel OR sfg.sgncmb, see help!');
elseif isfield(cfg, 'channel') && ~isfield(cfg, 'sgncmb') ,
    channelselect = 1;
elseif ~isfield(cfg, 'channel') && isfield(cfg, 'sgncmb') ,
    channelselect = 2;
end;

% check TheilerT input
if isfield(cfg, 'TheilerT'),
    if strcmp(cfg.TheilerT, 'ACT')
        exc_type = 1;
    else
        exc_type = 2;
    end
end;


% check the format of input vectors
if size(cfg.toi,1)>size(cfg.toi,2)
    cfg.toi=cfg.toi';
elseif size(cfg.dim,1)>size(cfg.dim,2)
    cfg.dim=cfg.dim';
elseif size(cfg.tau,1)>size(cfg.tau,2)
    cfg.tau=cfg.tau';
elseif size(cfg.predicttime_u,1)>size(cfg.predicttime_u,2)
    cfg.predicttime_u=cfg.predicttime_u';
elseif size(cfg.kth_neighbors,1)>size(cfg.kth_neighbors,2)
    cfg.kth_neighbors=cfg.kth_neighbors';
elseif size(cfg.TheilerT,1)>size(cfg.TheilerT,2)
    cfg.TheilerT=cfg.TheilerT';
end
% %
% % % check alternative of trial selection and the required related inputs
% % if strcmp(data.TEprepare.cfg.trialselect, 'ACT')
% %     cfg.actthrvalue = data.TEprepare.cfg.actthrvalue;
% %     cfg.minnrtrials = data.TEprepare.cfg.minnrtrials;
% % elseif strcmp(data.TEprepare.cfg.trialselect, 'range')
% %     cfg.minnrtrials = data.TEprepare.cfg.minnrtrials;
% %     cfg.trial_to = data.TEprepare.cfg.trial_to;
% % end


if nargin == 2;
    shifttest = 0;
%     multirun = 0;
elseif nargin == 3 && strcmp(varargin{1}, 'shifttest') ;
    shifttest = 1;
end


fprintf(' - ok');

% read data
% -------------------------------------------------------------------------
fprintf('\nRead data');

datamat=zeros(size(data.trial,2),size(data.trial{1},1),size(data.trial{1},2));
for ii = 1:size(data.trial,2)
    datamat(ii,:,:)=data.trial{ii};
end

% read time values of the data
% use rounding for later use of the 'find' command
if iscell(data.time)
    alltime=cell2mat(data.time(1));
else
    alltime=data.time;
end

% find correct indices for the samples in alltime/cfg.toi
% to be used later
% look in the time dimension of the data
timeindices=zeros(1,2);
for ii = 1:size(cfg.toi,2)
    [col]=find(abs(alltime - cfg.toi(ii))==min(abs(alltime - cfg.toi(ii))));
    timeindices(ii)=col;
end

% convert k value from ms to sampling points
dimu=zeros(1,size(cfg.predicttime_u,2));
for ii = 1:size(cfg.predicttime_u,2)
    dimu(ii)=round(cfg.predicttime_u(ii)/1000*data.fsample);
end

% create empty result structure
TEresult=[];

fprintf(' - ok');


% get values from cfg
% -------------------------------------------------------------------------
channelcombi=data.TEprepare.channelcombi ;
channelcombilabel=data.TEprepare.channelcombilabel ;
ACT=data.TEprepare.ACT;
trials=data.TEprepare.trials;
nrtrials=data.TEprepare.nrtrials;


% calculate number of points inside the time series used for advance and delay
% embedding and check whether there is a sufficient number of them left
% -------------------------------------------------------------------------
if strcmp(cfg.trialselect, 'ACT')
    multiplyact= min( [max(max(ACT)) cfg.actthrvalue] );
    mindatapoints = (timeindices(2)+1-timeindices(1))-(max(cfg.dim)-1)*max(cfg.tau)*multiplyact-max(dimu);
else
    mindatapoints = (timeindices(2)+1-timeindices(1))-(max(cfg.dim)-1)*max(cfg.tau)*max(max(ACT))-max(dimu);
end


if isfield(data, 'datatype')
    if strcmp(data.datatype, 'fMRI')
        minsamples = 50;
    else
        minsamples = 150;
    end
else
    minsamples = 150;
end

if mindatapoints <= minsamples
    disp(mindatapoints)
    error('\nTRANSFER ENTROPY ERROR: not enough data points left after embedding ');
end


% Change to directory containing mex files for the nearest neighbors search
[dir_mex] = TEarch(cfg);
cd(dir_mex);



% Check calculation time by calculating one TE with pessimistic values for
% tau, dim, .... etc
% -------------------------------------------------------------------------
if strcmp(cfg.calctime, 'yes')
    fprintf(strcat('\nCheck calculation time of TE. Please wait...'))
    maxtr=find(nrtrials==max(nrtrials));
    
    %get time of single TE calculation (pessimistic case)
    timetest = 0;
    for ii = 1:5
        tic
         if exc_type == 1
             % use either the old predictor from 'V'(icente)
             % or the new one 'V(icente)W(ibral)'
             if strcmp(data.TEprepare.cfg.TEcalctype,'V')
             [te, mi] = TEvalues(squeeze(datamat(trials(maxtr(1)),1,timeindices(1):timeindices(2))),squeeze(datamat(trials(maxtr(1)),2,timeindices(1):timeindices(2))),cfg.dim,round(cfg.tau*data.TEprepare.maxact),max(dimu),cfg.kth_neighbors,data.TEprepare.maxact);
             elseif strcmp(data.TEprepare.cfg.TEcalctype,'VW')
             [te, mi] = TECvalues(squeeze(datamat(trials(maxtr(1)),1,timeindices(1):timeindices(2))),squeeze(datamat(trials(maxtr(1)),2,timeindices(1):timeindices(2))),cfg.dim,round(cfg.tau*data.TEprepare.maxact),max(dimu),cfg.kth_neighbors,data.TEprepare.maxact);
             end
             elseif exc_type == 2
             if strcmp(data.TEprepare.cfg.TEcalctype,'V')    
             [te, mi] = TEvalues(squeeze(datamat(trials(maxtr(1)),1,timeindices(1):timeindices(2))),squeeze(datamat(trials(maxtr(1)),2,timeindices(1):timeindices(2))),cfg.dim,round(cfg.tau*data.TEprepare.maxact),max(dimu),cfg.kth_neighbors,cfg.TheilerT);
             elseif strcmp(data.TEprepare.cfg.TEcalctype,'VW')
              [te, mi] = TECvalues(squeeze(datamat(trials(maxtr(1)),1,timeindices(1):timeindices(2))),squeeze(datamat(trials(maxtr(1)),2,timeindices(1):timeindices(2))),cfg.dim,round(cfg.tau*data.TEprepare.maxact),max(dimu),cfg.kth_neighbors,cfg.TheilerT);
             end
         end
        timetest = timetest + toc;
    end
    timetest = timetest/5;
    
    % time * nr of loops
    % nrloops = size(cfg.sgncmb,1)*size(trials,2)*size(cfg.dim,2)*size(cfg.tau,2)*size(dimu,2)*size(cfg.kth_neighbors,2)*size(cfg.TheilerT,2);
    timeappr = timetest*size(channelcombi,1)*mean(nrtrials)*size(cfg.dim,2)*size(cfg.tau,2)*size(dimu,2)*size(cfg.kth_neighbors,2)*size(cfg.TheilerT,2);
    
    
    % if unshuffled and shuffeled data are calculated double the time
    if isfield(cfg, 'permtest')
        timeappr = timeappr*2;
    elseif isfield(cfg, 'permtest') && strcmp(cfg.shifttest, 'yes')
        timeappr = timeappr*3;
    elseif isfield(cfg, 'permtest') && isfield(cfg, 'NrSubjects')
        timeappr = timeappr*2*cfg.NrSubjects;
    elseif isfield(cfg, 'permtest') && strcmp(cfg.shifttest, 'yes') && isfield(cfg, 'NrSubjects')
        timeappr = timeappr*3*cfg.NrSubjects;
    end
    
    % time in minutes OR hours and days
    timehh = floor(timeappr/60^2);                      %hours
    if timehh<1
        timemm = floor(mod((timeappr/60), 60));         %minutes
        fprintf(strcat('\n!!! The calculation of TE takes appr. : ~',num2str(timemm),' minutes  !!!\n'));
    else
        fprintf(strcat('\n!!! The calculation of TE takes appr. : ~',num2str(timehh),' hours (',num2str(timehh/24),' days) !!!\n'));
    end
    
end

% Start calculation of TE
% -------------------------------------------------------------------------
fprintf('\nCalculating transfer entropy');


% prepare text waitbar
fprintf('\nPlease wait...\n');
for ii = 1:size(channelcombi,1)
    fprintf('-')
end
fprintf('\n')

% create zeros result matrices

TEresult.TEmat=zeros(size(channelcombi,1),size(cfg.predicttime_u,2),max(nrtrials));
TEresult.MImat=zeros(size(channelcombi,1),size(cfg.predicttime_u,2),max(nrtrials));

% loops for scanning channels with different parameter values for TE
for channelpair = 1:size(channelcombi,1)
    fprintf('-');
        
        
        for nt = 1:nrtrials(channelpair,1)
            
            trial1 = trials(channelpair,nt);
            if strcmp(cfg.shuffle, 'no')
                trial2 = trial1;
                timespan = timeindices(1):timeindices(2);
            elseif strcmp(cfg.shuffle, 'yes')
                
                if strcmp(cfg.surrogatetype, 'trialshuffling')
                    if mod(trials(channelpair,nt),nrtrials(channelpair,1)) == 0
                        trial2 = trials(channelpair,1);
                    else
                        trial2 = mod(trials(channelpair,nt),nrtrials(channelpair,1))+1;
                    end
                    timespan = timeindices(1):timeindices(2);
                    
                elseif strcmp(cfg.surrogatetype, 'blockresampling')
                    trial2 = trials(channelpair,1);
                    cutpoint = round( (timeindices(2)-timeindices(1)+1) * rand(1));
                    timespan =[cutpoint:timeindices(2),timeindices(1):cutpoint-1];
                    
                elseif strcmp(cfg.surrogatetype, 'trialreverse')
                    trial2 = trials(channelpair,1);
                    timespan = timeindices(1):timeindices(2);
                    flipdim(timespan,2)
                    
                elseif strcmp(cfg.surrogatetype, 'blockreverse1')
                    trial2 = trials(channelpair,1);
                    cutpoint = round( (timeindices(2)-timeindices(1)+1) * rand(1));
                    timespan =flipdim([cutpoint:timeindices(2),timeindices(1):cutpoint-1],2);
                    
                elseif strcmp(cfg.surrogatetype, 'blockreverse2')
                    trial2 = trials(channelpair,1);
                    cutpoint = round( (timeindices(2)-timeindices(1)+1) * rand(1));
                    timespan =[flipdim(cutpoint:timeindices(2),2),timeindices(1):cutpoint-1];
                    
                elseif strcmp(cfg.surrogatetype, 'blockreverse3')
                    trial2 = trials(channelpair,1);
                    cutpoint = round( (timeindices(2)-timeindices(1)+1) * rand(1));
                    timespan =[cutpoint:timeindices(2),flipdim(timeindices(1):cutpoint-1,2)];
                    
                end
                
            end
            
            for u=1:size(dimu,2)
                
                if shifttest == 1
                    if strcmp(cfg.shifttype, 'onesample')
                        a=squeeze(datamat(trial1,channelcombi(channelpair,1),timeindices(1)+1:timeindices(2)));
                        b=squeeze(datamat(trial2,channelcombi(channelpair,2),timeindices(1):timeindices(2)-1));
                    elseif strcmp(cfg.shifttype, 'predicttime')
                        a=squeeze(datamat(trial1,channelcombi(channelpair,1),timeindices(1)+dimu(u):timeindices(2)));
                        b=squeeze(datamat(trial2,channelcombi(channelpair,2),timeindices(1):timeindices(2)-dimu(u)));
                    end
                    
                else
                    a=squeeze(datamat(trial1,channelcombi(channelpair,1),timespan));
                    b=squeeze(datamat(trial2,channelcombi(channelpair,2),timespan));
                end
                
                if exc_type == 1
                    if strcmp(data.TEprepare.cfg.TEcalctype,'V')
                    [te, mi] = TEvalues(a,b,cfg.dim,round(cfg.tau*ACT(trials(nt),channelcombi(channelpair,2))),dimu(u),cfg.kth_neighbors,ACT(trials(nt),channelcombi(channelpair,2)));
                    elseif strcmp(data.TEprepare.cfg.TEcalctype,'VW')
                     [te, mi] = TECvalues(a,b,cfg.dim,round(cfg.tau*ACT(trials(nt),channelcombi(channelpair,2))),dimu(u),cfg.kth_neighbors,ACT(trials(nt),channelcombi(channelpair,2)));
                    end
                elseif exc_type == 2
                    if strcmp(data.TEprepare.cfg.TEcalctype,'V')
                    [te, mi] = TEvalues(a,b,cfg.dim,round(cfg.tau*ACT(trials(nt),channelcombi(channelpair,2))),dimu(u),cfg.kth_neighbors,cfg.TheilerT);
                    elseif strcmp(data.TEprepare.cfg.TEcalctype,'VW')
                     [te, mi] = TECvalues(a,b,cfg.dim,round(cfg.tau*ACT(trials(nt),channelcombi(channelpair,2))),dimu(u),cfg.kth_neighbors,cfg.TheilerT);
                    end
                end
                
                TEresult.TEmat(channelpair,u,nt)=te;
                TEresult.MImat(channelpair,u,nt)=mi;
                
            end
        end

end
fprintf('\nCalculation finished')



% results of unshuffled data
% -------------------------------------------------------------------------

if shifttest ==1
    TEresult.shifttest='yes';
end

if strcmp(cfg.shuffle, 'no')
    
    TEresult.act=ACT;
    TEresult.trials = trials; % save used trials in Result matrix
    TEresult.dimord = 'chanpair_u_trial';
    TEresult.cfg = cfg;
    if channelselect == 1
        TEresult.label=cfg.channel;
        TEresult.sgncmb=channelcombilabel;
    elseif channelselect == 2
        TEresult.sgncmb=cfg.sgncmb;
    end
    
    
end

fprintf('\n')

%% Returning to the working directory
cd(working_directory)

return;

